from datetime import date
from decimal import Decimal
from typing import Literal, Optional, Union

from pydantic import BaseModel, NonNegativeInt

from . import RecordType


class NormalizedRecordBase(
    BaseModel, allow_mutation=False, anystr_strip_whitespace=True, min_anystr_length=1
):
    record_type: RecordType
    record_id: str
    facility_id: NonNegativeInt


class Patient(NormalizedRecordBase):
    record_type: Literal[RecordType.Patient] = RecordType.Patient
    mrn: str
    first_name: str
    last_name: str
    middle_name: Optional[str]


class Reimbursement(NormalizedRecordBase):
    record_type: Literal[RecordType.Reimbursement] = RecordType.Reimbursement
    patient_mrn: str
    payer: str
    rate_text: str
    effective_from: date
    effective_to: Optional[date] = None
    daily_rate: Decimal = Decimal(0)


NormalizedRecordTypes = Union[Patient, Reimbursement]
